<?php

namespace Modules\Document\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use App\Utils\ModuleUtil;
use Illuminate\Support\Facades\DB;
use Modules\Document\Entities\Document;
use Modules\Document\Entities\DocumentShare;
use App\User;
use Modules\Document\Notifications\DocumentShared;
use Notification;
use Spatie\Permission\Models\Role;
use App\Category;

class DocumentController extends Controller
{
    /**
     * All Utils instance.
     *
     */
    protected $moduleUtil;

    /**
     * Constructor
     *
     * @param CommonUtil
     * @return void
     */
    public function __construct(ModuleUtil $moduleUtil)
    {
        $this->moduleUtil = $moduleUtil;
    }

    /**
     * Display a listing of the resource.
     * @return Response
     */
    public function index()
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'document_module') && auth()->user()->can('access.document')))) {
            abort(403, 'Unauthorized action.');
        }

        $user_id = request()->session()->get('user.id');
        $role_id = User::find($user_id)->roles()->first()->id;
        
        $module_data = $this->moduleUtil->getModuleData('getAssignedTaskForUser', $user_id);
        $todo_ids = $module_data['Essentials'];
        
        $query = Document::with(['createdBy', 'shares'])
                    ->where('business_id', $business_id)
                    ->leftJoin('document_file_shares as SSS', 'document_files.id', '=', 'SSS.document_file_id');

        if (!auth()->user()->can('superadmin')) {
            $query->where('created_by', auth()->user()->id)
                    ->orWhere(function ($query) use ($role_id) {
                        $query->where('SSS.shared_id', '=', $role_id)
                            ->where('SSS.shared_with', '=', 'role');
                    })
                    ->orwhere(function ($query) use ($user_id) {
                        $query->where('SSS.shared_id', '=', $user_id)
                            ->where('SSS.shared_with', '=', 'user');
                    });

            if (!empty($todo_ids)) {
                $query->orwhere(function ($query) use ($todo_ids) {
                        $query->whereIn('SSS.shared_id', $todo_ids)
                            ->where('SSS.shared_with', '=', 'todo');
                    });
            }
                    
        }

        $documents= $query->select('document_files.id', 'name', 'document_files.updated_at', 
                            'created_by', 'folder_id')
                        ->orderByDesc('document_files.updated_at')
                        ->groupBy('document_files.id')
                        ->get();

        $todos= [];
        if (!empty($module_data['Essentials'])) {
            $todos = $module_data['Essentials'];
        }

        $users = User::forDropdown($business_id, false);
        $roles = $this->moduleUtil->getDropdownForRoles($business_id);

        foreach($documents as $key => $value) {
            if(count($value->shares) > 0) {
                $shered_users = [];
                $shared_roles = [];
                $shared_todos = [];

                foreach($value->shares as $share) {
                    if($share->shared_with == 'user') {
                        $shered_users[] = $users[$share->shared_id] ?? '';
                    } else if($share->shared_with == 'role') {
                        $shared_roles[] = $roles[$share->shared_id] ?? '';
                    } else if($share->shared_with == 'role') {
                        $shared_todos[] = $todos[$share->shared_id] ?? '';
                    }
                }

                $documents[$key]->shered_users = $shered_users;
                $documents[$key]->shared_roles = $shared_roles;
                $documents[$key]->shared_todos = $shared_todos;
            }
        }

        $folders = Category::where('business_id', $business_id)
                        ->where('category_type', 'document')
                        ->orderBy('name', 'asc')
                        ->get();

        return view('document::doc.index')
            ->with(compact('documents', 'folders'));
    }

    /**
     * Show the form for creating a new resource.
     * @return Response
     */
    public function create()
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'document_module') && auth()->user()->can('create.document')))) {
            abort(403, 'Unauthorized action.');
        }
        $folder_id = request()->input('folder_id', null);
        return view('document::doc.create')->with(compact('folder_id'));
    }

    /**
     * Store a newly created resource in storage.
     * @param Request $request
     * @return Response
     */
    public function store(Request $request)
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'document_module') && auth()->user()->can('create.document')))) {
            abort(403, 'Unauthorized action.');
        }

        try {

            $input['name'] = !empty($request->input('name')) ? $request->input('name') : 'New Document';
            $input['doc_data'] = $request->input('doc_data');
            $input['business_id'] = $business_id;
            $input['created_by']= auth()->user()->id;
            $input['folder_id'] = $request->input('folder_id');
            DB::beginTransaction();

            Document::create($input); 

            DB::commit();
            $output = [
                'success' => true,
                'msg' => __('lang_v1.success')
            ];
        } catch (Throwable $e) {
            DB::rollBack();

            \Log::emergency("File:" . $e->getFile(). "Line:" . $e->getLine(). "Message:" . $e->getMessage());

            $output = [
                'success' => false,
                'msg' => __('messages.something_went_wrong')
            ];
        }

        return redirect()
            ->action('\Modules\Document\Http\Controllers\DocumentController@index')
            ->with('status', $output);
    }

    /**
     * Show the specified resource.
     * @param int $id
     * @return Response
     */
    public function show($id)
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || auth()->user()->can('create.document') || auth()->user()->can('access.document'))) {
            abort(403, 'Unauthorized action.');
        }

        $user_id = request()->session()->get('user.id');
        $role_id = User::find($user_id)->roles()->first()->id;
        
        $module_data = $this->moduleUtil->getModuleData('getAssignedTaskForUser', $user_id);
        $todo_ids = $module_data['Essentials'];
        
        $query = Document::where('business_id', $business_id)
                    ->where('document_files.id', $id)
                    ->leftJoin('document_file_shares as SSS', 'document_files.id', '=', 'SSS.document_file_id');

        if (!auth()->user()->can('superadmin')) {
            $query->where('created_by', auth()->user()->id)
                    ->orWhere(function ($query) use ($role_id) {
                        $query->where('SSS.shared_id', '=', $role_id)
                            ->where('SSS.shared_with', '=', 'role');
                    })
                    ->orwhere(function ($query) use ($user_id) {
                        $query->where('SSS.shared_id', '=', $user_id)
                            ->where('SSS.shared_with', '=', 'user');
                    });

            if (!empty($todo_ids)) {
                $query->orwhere(function ($query) use ($todo_ids) {
                        $query->whereIn('SSS.shared_id', $todo_ids)
                            ->where('SSS.shared_with', '=', 'todo');
                    });
            }
                    
        }

        $document = $query
                        ->select('document_files.business_id', 'document_files.id', 'document_files.name', 'document_files.doc_data', 'document_files.created_by')
                        ->firstOrFail();

        return view('document::doc.show')
            ->with(compact('document'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param int $id
     * @return Response
     */
    public function edit($id)
    {
        return view('document::edit');
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param int $id
     * @return Response
     */
    public function update(Request $request, $id)
    {
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || auth()->user()->can('create.document'))) {
            abort(403, 'Unauthorized action.');
        }

        try {

            $input['name'] = !empty($request->input('name')) ? $request->input('name') : 'My Document';
            $input['doc_data'] = $request->input('doc_data');

            $user_id = request()->session()->get('user.id');
            $role_id = User::find($user_id)->roles()->first()->id;

            $module_data = $this->moduleUtil->getModuleData('getAssignedTaskForUser', $user_id);
            $todo_ids = $module_data['Essentials'];

            DB::beginTransaction();
                $query = Document::where('business_id', $business_id)
                        ->where('document_files.id', $id)
                        ->leftJoin('document_file_shares as SSS', 'document_files.id', '=', 'SSS.document_file_id');

                if (!auth()->user()->can('superadmin')) {
                    $query->where('created_by', auth()->user()->id)
                            ->orWhere(function ($query) use ($role_id) {
                                $query->where('SSS.shared_id', '=', $role_id)
                                    ->where('SSS.shared_with', '=', 'role');
                            })
                            ->orwhere(function ($query) use ($user_id) {
                                $query->where('SSS.shared_id', '=', $user_id)
                                    ->where('SSS.shared_with', '=', 'user');
                            });

                    if (!empty($todo_ids)) {
                        $query->orwhere(function ($query) use ($todo_ids) {
                                $query->whereIn('SSS.shared_id', $todo_ids)
                                    ->where('SSS.shared_with', '=', 'todo');
                            });
                    }
                            
                }

                $document = $query
                                ->select('document_files.business_id', 'document_files.id', 'document_files.name', 'document_files.doc_data')
                                ->firstOrFail();

                if (!empty($document)) {
                    Document::where('business_id', $business_id)
                        ->where('id', $id)
                        ->update($input);
                }

            DB::commit();
            $output = [
                'success' => true,
                'msg' => __('lang_v1.success')
            ];
        } catch (Throwable $e) {
            DB::rollBack();

            \Log::emergency("File:" . $e->getFile(). "Line:" . $e->getLine(). "Message:" . $e->getMessage());

            $output = [
                'success' => false,
                'msg' => __('messages.something_went_wrong')
            ];
        }

        return redirect()
            ->action('\Modules\Document\Http\Controllers\DocumentController@show', [$id])
            ->with('status', $output);
    }

    /**
     * Remove the specified resource from storage.
     * @param int $id
     * @return Response
     */
    public function destroy($id)
    {
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || auth()->user()->can('create.document'))) {
            abort(403, 'Unauthorized action.');
        }

        if (request()->ajax()) {
            try {
                DB::beginTransaction();
                
                Document::where('business_id', $business_id)
                    ->where('created_by', auth()->user()->id)
                    ->where('id', $id)
                    ->delete();

                DB::commit();
                $output = [
                    'success' => true,
                    'msg' => __('lang_v1.success')
                ];
            } catch (Throwable $e) {
                DB::rollBack();

                \Log::emergency("File:" . $e->getFile(). "Line:" . $e->getLine(). "Message:" . $e->getMessage());

                $output = [
                    'success' => false,
                    'msg' => __('messages.something_went_wrong')
                ];
            }
        }

        return $output;
    }

    public function getShareDocument(Request $request, $id)
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'document_module')))) {
            abort(403, 'Unauthorized action.');
        }

        if ($request->ajax()) {
            $module_data = $this->moduleUtil->getModuleData('getTodosDropdown', $business_id);

            $todos= [];
            if (!empty($module_data['Essentials'])) {
                $todos = $module_data['Essentials'];
            }

            $users = User::forDropdown($business_id, false);
            $roles = $this->moduleUtil->getDropdownForRoles($business_id);
            
            $shared_documents = DocumentShare::where('document_file_id', $id)
                                    ->get()
                                    ->groupBy('shared_with');

            $shared_todos = [];
            $shared_roles = [];
            $shared_users = [];
            if (!empty($shared_documents['todo'])) {
                $shared_todos = $shared_documents['todo']->pluck('shared_id')->toArray();
            }

            if (!empty($shared_documents['role'])) {
                $shared_roles = $shared_documents['role']->pluck('shared_id')->toArray();
            }

            if (!empty($shared_documents['user'])) {
                $shared_users = $shared_documents['user']->pluck('shared_id')->toArray();
            }

            return view('document::doc.partials.share_doc')
                ->with(compact('todos', 'id', 'shared_roles', 'roles', 'users', 'shared_todos', 'shared_users'));
        }

    }

    public function postShareDocument(Request $request)
    {   
        $business_id = request()->session()->get('user.business_id');

        if (!(auth()->user()->can('superadmin') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'document_module')))) {
            abort(403, 'Unauthorized action.');
        }

        if ($request->ajax()) {
            try {
                $doc_id = $request->input('doc_id');
                $shares = $request->input('share');
                DB::beginTransaction();

                    //Update shared documents based on selected data
                    $notify_to_shared_ids = []; //send notification to users
                    if (!empty($shares)) {
                        foreach ($shares as $shared_with => $shared_ids) {
                            $existing_shared_ids = [];
                            foreach ($shared_ids as $id) {
                                $existing_shared_ids[] = $id;
                                $data = [
                                    'document_file_id' => $doc_id,
                                    'shared_with' => $shared_with,
                                    'shared_id' => $id
                                ];

                                $doc = DocumentShare::firstOrCreate($data, $data);

                                if ($doc->shared_with == 'user' && $doc->wasRecentlyCreated) {
                                    $notify_to_shared_ids[] = $doc->shared_id;
                                }

                                //get users of the role to notify
                                if ($doc->shared_with == 'role' && $doc->wasRecentlyCreated) {

                                    $role = Role::where('business_id', $business_id)
                                        ->find($doc->shared_id);
                                    $users = User::role($role->name)->get()->pluck('id')->toArray();
                                    $notify_to_shared_ids = array_unique(array_merge($notify_to_shared_ids,$users));

                                }
                            }
                            DocumentShare::where('document_file_id', $doc_id)
                                ->where('shared_with', $shared_with)
                                ->whereNotIn('shared_id', $existing_shared_ids)
                                ->delete();
                        }
                    }

                    //if shared with is empty then delete whole rows of given doc & shared with
                    if (empty($shares['todo'])) {
                        DocumentShare::where('document_file_id', $doc_id)
                            ->where('shared_with', 'todo')
                            ->delete();
                    } 

                    if (empty($shares['user'])) {
                        DocumentShare::where('document_file_id', $doc_id)
                            ->where('shared_with', 'user')
                            ->delete();
                    }

                    if (empty($shares['role'])) {
                        DocumentShare::where('document_file_id', $doc_id)
                            ->where('shared_with', 'role')
                            ->delete();
                    }

                    if (!empty($notify_to_shared_ids)) {
                        $this->notifyUsersOfSharedDocs($business_id, $doc_id, $notify_to_shared_ids);
                    }
                DB::commit();
                $output = [
                    'success' => true,
                    'msg' => __('lang_v1.success')
                ];
            } catch (Throwable $e) {
                DB::rollBack();

                \Log::emergency("File:" . $e->getFile(). "Line:" . $e->getLine(). "Message:" . $e->getMessage());

                $output = [
                    'success' => false,
                    'msg' => __('messages.something_went_wrong')
                ];
            }

            return $output;
        }
    }

    public function notifyUsersOfSharedDocs($business_id, $doc_id, $shared_ids)
    {   
        $users = User::where('business_id', $business_id)
                    ->find($shared_ids);

        Notification::send($users, new DocumentShared($doc_id));
    }

    public function addFolder(Request $request)
    {
        try {
            $input = $request->only(['name']);
            if(!empty($request->input('folder_id'))) {
                Category::where('business_id', $request->session()->get('user.business_id'))
                        ->where('id', $request->input('folder_id'))
                        ->where('category_type', 'document')
                        ->update($input);
            } else {
                $input['parent_id'] = 0;
                $input['business_id'] = $request->session()->get('user.business_id');
                $input['created_by'] = $request->session()->get('user.id');
                $input['category_type'] = 'document';

                Category::create($input);
            }
            
            $output = ['success' => true,
                            'msg' => __("lang_v1.success")
                        ];
        } catch (\Throwable $e) {
            \Log::emergency("File:" . $e->getFile(). "Line:" . $e->getLine(). "Message:" . $e->getMessage());
            
            $output = ['success' => false,
                            'msg' => __("messages.something_went_wrong")
                        ];
        }

        return redirect()
                ->action('\Modules\Document\Http\Controllers\DocumentController@index')
                ->with('status', $output);
    }

    public function moveToFolder(Request $request)
    {
        if(!empty($request->input('move_to_folder'))) {
            $business_id = $request->session()->get('user.business_id');

            Document::where('business_id', $business_id)
                    ->where('created_by', auth()->user()->id)
                    ->where('id', $request->input('document_id'))
                    ->update(['folder_id' => $request->input('move_to_folder')]);

            $output = ['success' => true,
                    'msg' => __("lang_v1.success")
                ];

            return redirect()
                ->action('\Modules\Document\Http\Controllers\DocumentController@index')
                ->with('status', $output);

        }
    }
}
